package com.hai.service.impl;

import com.hai.dao.HighChildOrderMapper;
import com.hai.dao.HighOrderMapper;
import com.hai.entity.HighChildOrder;
import com.hai.entity.HighChildOrderExample;
import com.hai.entity.HighOrder;
import com.hai.entity.HighOrderExample;
import com.hai.service.HighOrderService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/26 23:06
 */
@Service("highOrderService")
public class HighOrderServiceImpl implements HighOrderService {

    @Resource
    private HighOrderMapper highOrderMapper;

    @Resource
    private HighChildOrderMapper highChildOrderMapper;

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void insertOrder(HighOrder highOrder) {
        highOrderMapper.insert(highOrder);

        for (HighChildOrder childOrder : highOrder.getHighChildOrderList()) {
            childOrder.setOrderId(highOrder.getId());
            highChildOrderMapper.insert(childOrder);
        }
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void updateOrder(HighOrder highOrder) {
        highOrderMapper.updateByPrimaryKey(highOrder);

        for (HighChildOrder childOrder : highOrder.getHighChildOrderList()) {
            highChildOrderMapper.updateByPrimaryKey(childOrder);
        }
    }

    @Override
    public HighOrder getOrderById(Long id) {
        HighOrder order = highOrderMapper.selectByPrimaryKey(id);
        if (order == null) {
            return null;
        }

        HighChildOrderExample example = new HighChildOrderExample();
        example.createCriteria().andOrderIdEqualTo(order.getId());
        order.setHighChildOrderList(highChildOrderMapper.selectByExample(example));

        return order;
    }

    @Override
    public HighOrder getOrderByOrderNo(String orderNo) {
        HighOrderExample example = new HighOrderExample();
        example.createCriteria().andOrderNoEqualTo(orderNo);

        List<HighOrder> list = highOrderMapper.selectByExample(example);
        if (list != null && list.size() > 0) {
           return getOrderById(list.get(0).getId());
        }
        return null;
    }

    @Override
    public List<HighOrder> getOrderList(Map<String, Object> map) {
        HighOrderExample example = new HighOrderExample();
        HighOrderExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "memId") != null) {
            criteria.andMemIdEqualTo(MapUtils.getLong(map, "memId"));
        }

        if (MapUtils.getInteger(map, "status") != null) {
            criteria.andOrderStatusEqualTo(MapUtils.getInteger(map, "status"));
        }

        if (MapUtils.getString(map, "orderNo") != null) {
            criteria.andOrderNoEqualTo(MapUtils.getString(map, "orderNo"));
        }

        example.setOrderByClause("create_time desc");
        return highOrderMapper.selectByExample(example);
    }
}
